unit DirectDraw;

interface
{$Z4}{$A+}
uses Forms, SysUtils,  OLE2, Windows , DDraw, D3DTypes, Graphics, classes;



type
  TDDScreen = class ( TObject )
  private
    FFrontSurface, FBackSurface: IDirectDrawSurface ;
    FiWidth, FiHeight : integer;
    FFullScreen : boolean;

    mhForm : HWND;
    mrcFront : TRect;

    mClipper : IDirectDrawClipper;
  public
    DDraw: IDirectDraw ;
    bSWRender : Boolean;
    bHasColorKey, bHasOverlay, bcanStretch : boolean;

    constructor Create ( Width, Height, Depth: DWORD ; hform : HWND; bFullScreen : boolean) ;
    destructor Destroy ; override ;
    procedure Flip ;
    procedure WindowMove;

    property FrontSurface: IDirectDrawSurface read FFrontSurface ;
    property BackSurface: IDirectDrawSurface read FBackSurface ;
    property width : integer read FiWIdth;
    property height : integer read FiHeight;
    property FullScreen : boolean read FFullScreen;
  end ;

implementation


var
  DDResult: HRESULT ;
  DDPalette: IDirectDrawPalette ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.CREATE
 |
 | Set up DirectDrawSurface objects
 |
 -----------------------------------------------------------}

constructor TDDScreen.Create ( Width, Height, Depth: DWORD ; hform : HWND; bFullScreen : boolean) ;
const
  FullSurfaceDesc: DDSURFACEDESC = (
      dwSize: sizeof ( FullSurfaceDesc ) ;
      dwFlags: DDSD_CAPS + DDSD_BACKBUFFERCOUNT ;
      dwBackBufferCount: 1 ;
      ddsCaps: ( dwCaps: DDSCAPS_PRIMARYSURFACE + DDSCAPS_FLIP + DDSCAPS_COMPLEX + DDSCAPS_3DDEVICE )
      ) ;
  WinSurfaceDesc : DDSURFACEDESC = (
      dwSize : sizeof( WinSurfaceDesc);
      dwFlags : DDSD_CAPS;
      ddsCaps : (dwCaps : DDSCAPS_PRIMARYSURFACE);
      ) ;
var
  BackCaps: DDSCAPS ;
  i : Integer ;
  PEntry: Array [ 0..255 ] of TPaletteEntry ;
  sd : DDSURFACEDESC;
  hdcPal : HDC;
  dd_Caps, dd_HELCaps : DDCAPS;
begin
  inherited Create ;

  FIWidth := width;
  FIHeight := Height;

  mhForm := hForm;

  { Initialize the DirectDraw system }

  DDResult := DirectDrawCreate ( nil, DDraw, nil ) ;
  if DDResult <> DD_OK then
    raise Exception.Create (
        Format ( 'DirectDrawCreate failed: %x', [ DDResult ] ) ) ;

  DDresult := DDraw.GetCaps(dd_caps, dd_HELCaps );

//     /* Determine if the hardware supports overlay surfaces */
  if ((dd_caps.dwCaps and DDCAPS_OVERLAY) <> 0) then
     bHasOverlay := True
  else
      bHasOverlay := False;

//     /* Determine if the hardware supports colorkeying */
  if ((dd_caps.dwCaps and DDCAPS_COLORKEY) <> 0 ) then
     bHasColorKey := True
  else
      bHasColorKey := False;

//     /* Determine if the hardware supports scaling of the overlay surface */
  if ((dd_caps.dwCaps and DDCAPS_OVERLAYSTRETCH) <> 0 ) then
     bCanStretch := True
  else
      bCanStretch := False;

  { Set no cooperation with other applications }

  if bFullScreen then
     DDResult := DDraw.SetCooperativeLevel ( hForm,
      DDSCL_FULLSCREEN +DDSCL_NOWINDOWCHANGES+DDSCL_ALLOWREBOOT +DDSCL_EXCLUSIVE )     //DDSCL_ALLOWMODEX+DDSCL_NOWINDOWCHANGES +DDSCL_ALLOWREBOOT +
  else
      DDResult := DDraw.SetCooperativeLevel (hForm, DDSCL_NORMAL);

  if DDResult <> DD_OK then
    raise Exception.Create ( Format (
        'IDirectSound.SetCooperativeLevel failed: %x', [ DDResult ] ) ) ;

  if bFullScreen then
     begin
  { Set display mode }

        DDResult := DDraw.SetDisplayMode ( Width, Height, Depth ) ;
        if DDResult <> DD_OK then
        raise Exception.Create ( Format (
              'IDirectDraw.SetDisplayMode failed: %x', [ DDResult ] ) ) ;

  { Create triple-buffered backing store }

        DDResult := DDraw.CreateSurface ( FullSurfaceDesc, FFrontSurface, nil ) ;
        if DDResult <> DD_OK then
        raise Exception.Create ( Format (
              'IDirectDraw.CreateSurface failed: %x', [ DDResult ] ) ) ;

        BackCaps.dwCaps := DDSCAPS_BACKBUFFER ;
        DDResult := FFrontSurface.GetAttachedSurface ( BackCaps, FBackSurface ) ;
        if DDResult <> DD_OK then
        raise Exception.Create ( Format (
              'IDirectDrawSurface.GetAttachedSurface failed: %x', [ DDResult ] ) ) ;
     end
  else
      begin
           DDResult := DDraw.CreateSurface (WinSurfaceDesc, FFrontSurface, nil);
           if DDResult <> DD_OK then
           raise Exception.Create ( Format (
              'Windowed IDirectDraw.CreateSurface failed: %x', [ DDResult ] ) ) ;

           ZeroMemory(@sd, sizeof(sd));
           sd.dwSize := sizeof(sd);
           sd.dwFlags := DDSD_WIDTH + DDSD_HEIGHT + DDSD_CAPS;
           sd.dwHeight := Height;
           sd.dwWidth := Width;
           sd.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN + DDSCAPS_3DDEVICE;

           DDResult := DDraw.CreateSurface (sd, FBackSurface, nil);
           if DDResult <> DD_OK then
           raise Exception.Create ( Format (
              'WinBack IDirectDraw.CreateSurface failed: %x', [ DDResult ] ) ) ;

           DDResult := DDraw.CreateClipper(0, mClipper, nil);
           if DDResult <> DD_OK then
           raise Exception.Create ( Format (
              'IDirectDraw.CreateClipper failed: %x', [ DDResult ] ) ) ;

           DDResult := mClipper.SetHwnd(0,hForm);
           if DDResult <> DD_OK then
           raise Exception.Create ( Format (
              'Clipper.SetHwnd failed: %x', [ DDResult ] ) ) ;

           DDResult := FFrontSurface.SetClipper(mClipper);
           if DDResult <> DD_OK then
           raise Exception.Create ( Format (
              'Surface.SetClipper failed: %x', [ DDResult ] ) ) ;

      end;

  bSWRender := TRUE;
  FBackSurface.GetSurfaceDesc(sd);
  if (sd.ddsCaps.dwcaps and DDSCAPS_VIDEOMEMORY) <> 0 then
     bSWRender := FALSE;

  if (sd.ddpfPixelFormat.dwRGBBitCount < 16) then
         begin
              hdcPal := GetDC(0);
              GetSystemPaletteEntries(hdcPal,0,256,pEntry);
              ReleaseDC(0, hdcPal);

              if (bFullScreen) then
                 begin
			pEntry[0].peFlags := D3DPAL_READONLY;
			for i := 1 to  254 do
                             pEntry[i].peFlags := PC_RESERVED;

			pEntry[255].peFlags := D3DPAL_READONLY;
                 end
              else
                  begin               //9
			for i := 0 to 9 do
                             pEntry[i].peFlags := D3DPAL_READONLY;
			for i := 10 to 245 do
                             pEntry[i].peFlags := PC_RESERVED;

			for i := 246 to 255 do
                             pEntry[i].peFlags := D3DPAL_READONLY;
                  end;
         end;

                                //+ DDPCAPS_ALLOW256
  DDraw.CreatePalette ( DDPCAPS_8BIT + DDPCAPS_INITIALIZE , @PEntry[0], DDPalette, nil ) ;
  FFrontSurface.SetPalette ( DDPalette ) ;
  FBackSurface.SetPalette (DDPalette);

  GetClientRect(mhform,mrcFront);
  ClientToScreen(mhForm,mrcFront.TopLeft);
  ClientToScreen(mhForm,mrcFront.BottomRight);

  FFullScreen := bFullScreen;
end ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.DESTROY
 |
 | Release DirectDrawSurface objects
 |
 -----------------------------------------------------------}

destructor TDDScreen.Destroy ;
begin
  if mClipper <> nil then
     mClipper.Release;
  mClipper := nil;
  if FFrontSurface <> nil then
    FFrontSurface.Release ;
  FFrontSurface := nil ;
  if DDraw <> nil then
    DDraw.Release ;
  DDraw := nil ;
  inherited Destroy ;
end ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.FLIP
 |
 | Flip to a new display buffer
 |
 -----------------------------------------------------------}

procedure TDDScreen.Flip ;
begin
  if FFullScreen then
     FFrontSurface.Flip ( nil, DDFLIP_WAIT )
  else
      FFrontSurface.Blt(mrcFront, FBackSurface, Rect(0,0,FIWidth,FIHeight), DDBLT_WAIT, nil);

end ;


procedure TDDScreen.WindowMove;
begin
  if mhForm = 0 then exit;
  
  GetClientRect(mhform,mrcFront);
  ClientToScreen(mhForm,mrcFront.TopLeft);
  ClientToScreen(mhForm,mrcFront.BottomRight);
end;

{-----------------------------------------------------------
 |
 | INITIALIZATION
 |
 | << ? >>
 |
 -----------------------------------------------------------}

initialization
begin
end ;

{-----------------------------------------------------------
 |
 | FINALIZATION
 |
 | << ? >>
 |
 -----------------------------------------------------------}

finalization
begin

end ;

end.
