unit TLTest;

interface
// Code by Massimo Bardetti - bigben@dircon.co.uk  (12 March 1997)
//
// Problems :
// WindowMoves are not detected by the component
// Window resizes are but once the screen is resized the animation is slower
// This is because every resize I should re-create the front and Back surface
// instead of only updating the rect information.
// Use arrow keys to translate the camera
// Use Shift+arrow keys to rotate the camera
// Use F2 to add more objects to the scene
// Use Esc or F1 to stop
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, TDirect3D, TExecute, D3D, D3dTypes, D3DCaps, DDraw, TMatrix,
  StdCtrls;

type
  TForm1 = class(TForm)
    Direct3DIM1: TDirect3DIM;
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    CheckScreen: TCheckBox;
    Edit1: TEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AppKeyDown(var Msg: TMsg; var Handled: Boolean);
    procedure AddObject(EBRoot : TExecuteBuffer; xoff, yoff, zoff : single; rspeed : single);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure GenerateSphere(sphere_r : single;num_rings : integer; num_sections : integer; sx : single; sy : single; sz : single);

const
 D2R : double = 0.01745329251994;
 NEWOBJ : integer = -1;
var
  Form1: TForm1;
  EB : TExecuteBuffer;
  lstVertices, lstTriangles : TList;
  mRot, mProjection, mView : T3DMatrix;
  bKeyend, bKeyProcess, bKeyShift : Boolean;
  j, iRot : Integer;

implementation

{$R *.DFM}

procedure TForm1.SpeedButton1Click(Sender: TObject);
var
   i : Integer;
   ry, siny, cosy : single;
begin
     lstVertices.clear;
     lstTriangles.clear;

     // Set Screen Dimensions in case we go FullScreen
     Direct3DIM1.iScreenWidth := 640;
     Direct3DIM1.iScreenHeight := 480;
     Direct3DIM1.iBpp := 8;
     // The Application handle is what is working for me to pass to the SetMode function
     Direct3DIM1.iApplicationHandle := Application.Handle;

     // This is all you have to do to create a DDraw surface with a Z Buffer attached and a Viewport
     // + all the rest you need to implement a Immediate Mode application
     Direct3DIM1.Initialize(D3DCOLOR_RGB, CheckScreen.Checked);

     // Create a Material and assign it to the background of the viewport
     Direct3dIM1.Material(NEWOBJ);
     Direct3DIM1.Material(0).SetColor(0.7, 0.7, 0.7);
     Direct3DIM1.Material(0).SetShades(1);
     Direct3DIM1.Direct3D.D3DViewport.SetBackGround(Direct3DIM1.Material(0).GetHandle);

     // Create Root Execute Buffer - It will be executed only once at the beginning of
     // the animation loop
     EB := Direct3DIM1.AddExecuteBuffer(nil,1024);
     EB.AddRenderState (D3DRENDERSTATE_FILLMODE, DWORD(D3DFILL_SOLID));
     EB.AddRenderState (D3DRENDERSTATE_SHADEMODE, DWORD (D3DSHADE_GOURAUD));
     EB.AddRenderState (D3DRENDERSTATE_DITHERENABLE, 1);
     EB.AddRenderState (D3DRENDERSTATE_ZENABLE, 1);

     mProjection := Direct3DIM1.matrix(NEWOBJ);
     mProjection._11 := 2.0;
     mProjection._22 := 2.0;
     mProjection._34 := 1.0;
     mProjection._43 := -1.0;
     mProjection._44 := 0.0;
     mProjection.Update();
     EB.AddState(D3DTRANSFORMSTATE_PROJECTION, mProjection.GetHandle());

     mView := Direct3DIM1.Matrix(NEWOBJ);
     mView._43 := 15;
     mView._42 := -3;
     ry := -20;
     siny := sin(ry*D2R);
     cosy := cos(ry*D2R);
     mView._22 := cosy;
     mView._23 := siny;
     mView._32 := -siny;
     mView._33 := cosy;
     mView.Update();
     EB.AddState(D3DTRANSFORMSTATE_VIEW, mView.GetHandle());
     EB.AddAmbLight($60606060);

     // Add one light to the viewport
     Direct3DIM1.Light(NEWOBJ);
     Direct3DIM1.Light(0).SetColor(1.5, 1.5, 1.5);
     Direct3DIM1.Light(0).SetType(D3DLIGHT_DIRECTIONAL);
     Direct3DIM1.Light(0).SetDirection(1,-1,1);
     Direct3DIM1.Light(0).Update;

     Direct3DIM1.Direct3D.D3DViewPort.AddLight(Direct3DIM1.Light(0).getInterface);

     // Create Material valid for all objects
     Direct3DIM1.Material(NEWOBJ);
     Direct3DIM1.Material(1).SetShades(8);
     Direct3DIM1.Material(1).SetColor(1,1,0);

     // Generate a list of vertices and triangles to describe a sphere
     GenerateSphere(1.5,15,15,0.8,0.5,1);

     // Add Another EB to set View Transofrmation Matrix and a MatMul operation to transform it
     // This will act as a camera object
     EB := Direct3DIM1.AddExecuteBuffer(EB,512);
     mRot := Direct3DIM1.Matrix(NEWOBJ);
     EB.AddMatMul(mView.GetHandle, mView.GetHandle, mRot.GetHandle);
     EB.AddMaterial(Direct3DIM1.Material(1).getHandle);

     // Create a number of objects
     // j represent the z offset from one object to the next
     // iRot describe the amount of rotation (or speed)
     j:=0;
     for iRot:=1 to StrToInt(Edit1.Text) do
         begin
              AddObject(EB,0,0,j, iRot);

              j:=j+4;
         end;

     iRot := 5;

     // Execute all the buffers
     bKeyend := False;
     bkeyProcess := True;
     while (Not bkeyEnd) do
           begin
              application.processmessages;
              Direct3DIM1.Execute;
          end;

     bKeyProcess := False;

     // Close all
     Direct3DIM1.Finish;

     i:=0;
     while i < lstVertices.Count do
       begin
           FreeMem(lstVertices.Items[i]);
           i:=i+1;
       end;

     i:=0;
     while i < lstTriangles.Count do
       begin
           FreeMem(lstTriangles.Items[i]);
           i:=i+1;
       end;

end;


// This function add two execute buffers to the EB passed in the parameters
// You can offset the object and set the speed of rotation
// The final EB tree structure will look like :
//
//
//                     EB     (First EB executed only once)
//                      |
//                     EB     (Second EB set view trans and material)
//                      |
//            ---------------------
//            |      |      |      |
//            EB     EB     EB     EB    (Third level of EBs add world transf and first obj)
//            |      |      |      |
//            EB     EB     EB     EB    (Fourth level of EBs add second object)
//
//
//  The execution will start from the root and will always follow the leftmost branch EB

procedure Tform1.AddObject(EBRoot : TExecuteBuffer; xoff, yoff, zoff : single; rspeed : single);
var
   EB2 : TExecuteBuffer;
   mWorld, mRot2, mRot3, mShad : T3DMatrix;
   i : integer;
   ry, cosy, siny : single;
begin

     // Add New EB
     EB2 := Direct3DIM1.AddExecuteBuffer(EBRoot,(lstVertices.count*sizeof(D3DVERTEX))+(lstTriangles.count*sizeof(D3DTRIANGLE))+1024);

     // Add vertices to it
     EB2.AddVertices(lstvertices.list, lstVertices.count);

     // Set world transformation matrix for this object
     mWorld := Direct3DIM1.matrix(NEWOBJ);
     EB2.AddState(D3DTRANSFORMSTATE_WORLD, mWorld.GetHandle());

     // Add some rotations matrices
     mRot2 := Direct3DIM1.Matrix(NEWOBJ);
     ry := rspeed;
     siny := sin(ry * D2R);
     cosy := cos(ry * D2R);
     mRot2._11 := cosy;
     mRot2._13 := -siny;
     mRot2._31 := siny;
     mRot2._33 := cosy;
     mRot2.UpDate();

     mRot3 := Direct3DIM1.Matrix(NEWOBJ);
     mrot3._41 := xoff;
     mrot3._42 := yoff;
     mrot3._43 := zoff;
     mrot3.Update();

     mShad := Direct3DIM1.Matrix(NEWOBJ);
     mShad._41 := -3.5;
     mShad._42 := -1.5;
     mShad._43 := -1.5;
     mShad.Update();

     // Add rotation matrices as Multipliers
     EB2.AddMatMul(mShad.getHandle(), mShad.GetHandle(), mRot2.getHandle());
     EB2.AddMatMul(mWorld.getHandle(), mShad.GetHandle(), mRot3.getHandle());

     // Process vertices
     EB2.AddProcess(lstVertices.count, 0, D3DPROCESSVERTICES_TRANSFORMLIGHT + D3DPROCESSVERTICES_UPDATEEXTENTS);

     // Add triangles
     EB2.AddTriangleOperation(lstTriangles.count);
     for i:=0 to lstTriangles.count - 1 do
         EB2.AddTriangleData(D3DTRIANGLE (lstTriangles[i]^));

////////////
     // Add new EB to previous one(without world transf. matrix this time)
     EB2 := Direct3DIM1.AddExecuteBuffer(EB2,(lstVertices.count*sizeof(D3DVERTEX))+(lstTriangles.count*sizeof(D3DTRIANGLE))+1024);

     // Offset vertices so that it will be visible !! etc. etc.
     for i := 0 to lstVertices.count-1 do
       begin
         D3DVERTEX (lstVertices.Items[i]^).x := D3DVERTEX (lstVertices.Items[i]^).x - 2.5;
         D3DVERTEX (lstVertices.Items[i]^).z := D3DVERTEX (lstVertices.Items[i]^).z - 2.5;
       end;

     EB2.AddVertices(lstvertices.list, lstVertices.count);

     EB2.AddProcess(lstVertices.count, 0, D3DPROCESSVERTICES_TRANSFORMLIGHT + D3DPROCESSVERTICES_UPDATEEXTENTS);

     EB2.AddTriangleOperation(lstTriangles.count);
     for i:=0 to lstTriangles.count - 1 do
         EB2.AddTriangleData(D3DTRIANGLE (lstTriangles[i]^));

      for i := 0 to lstVertices.count-1 do
       begin
         D3DVERTEX (lstVertices.Items[i]^).x := D3DVERTEX (lstVertices.Items[i]^).x + 2.5;
         D3DVERTEX (lstVertices.Items[i]^).z := D3DVERTEX (lstVertices.Items[i]^).z + 2.5;
       end;
end;

// Process key pressed , same as the other demo
procedure TForm1.AppKeyDown(var Msg: TMsg; var Handled: Boolean);
var
   ry,siny,cosy : single;
begin

if not bKeyProcess then exit;

if Msg.message = WM_KEYDOWN then
   begin
    Handled := True;

    case Msg.wParam of
     VK_SHIFT : bKeyShift := True;
     VK_UP: if (bKeyShift) then
              begin
                ry := ry + 0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._22 := cosy;
                mRot._32 := -siny;
                mRot._23 := siny;
                mRot._33 := cosy;
              end
            else
               mRot._43 := 0.2;
     VK_DOWN: if (bKeyShift) then
              begin
                ry := ry -0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._22 := cosy;
                mRot._32 := -siny;
                mRot._23 := siny;
                mRot._33 := cosy;
              end
            else
               mRot._43 := -0.2;
     VK_LEFT: if (bKeyShift) then
              begin
                ry := ry -0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._11 := cosy;
                mRot._21 := -siny;
                mRot._12 := siny;
                mRot._22 := cosy;
              end
            else
               mRot._41 := -0.2;
     VK_RIGHT: if (bKeyShift) then
              begin
                ry := ry + 0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._11 := cosy;
                mRot._21 := -siny;
                mRot._12 := siny;
                mRot._22 := cosy;
              end
            else
               mRot._41 := 0.2;

      VK_PRIOR: if (bKeyShift) then
              begin
                ry := ry -0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._11 := cosy;
                mRot._13 := -siny;
                mRot._31 := siny;
                mRot._33 := cosy;
              end
            else
               mRot._42 := -0.2;

     VK_NEXT: if (bKeyShift) then
              begin
                ry := ry + 0.5;
                siny := sin(ry * D2R);
                cosy := cos(ry * D2R);
                mRot._11 := cosy;
                mRot._13 := -siny;
                mRot._31 := siny;
                mRot._33 := cosy;
              end
            else
               mRot._42 := 0.2;

     VK_F2: begin
            AddObject(EB,0,0,j,iRot);
            j := j + 4;
            iRot := iRot + 1;
            end;


     VK_F1, VK_ESCAPE : bKeyEnd := True;
    end;
    mRot.Update;
   end
else if Msg.Message = WM_KEYUP then
     begin
       if (msg.wParam <> VK_SHIFT) then
        begin
          Handled := True;
          ry := 0;
          mRot._11 := 1;
          mRot._12 := 0;
          mRot._13 := 0;
          mRot._14 := 0;

          mRot._21 := 0;
          mRot._22 := 1;
          mRot._23 := 0;
          mRot._24 := 0;

          mRot._31 := 0;
          mRot._32 := 0;
          mRot._33 := 1;
          mRot._34 := 0;

          mRot._41 := 0;
          mRot._42 := 0;
          mRot._43 := 0;
          mRot._44 := 1;

          mRot.Update;
        end
       else
           bKeyShift := False;
     end;

end;


procedure TForm1.FormCreate(Sender: TObject);
begin
     bKeyprocess := false;
     lstvertices := TList.Create;
     lstTriangles := TList.Create;
     Application.OnMessage := AppKeyDown;
end;

// Generate a sphere
procedure GenerateSphere(sphere_r : single;num_rings : integer; num_sections : integer; sx : single; sy : single; sz : single);
var
   theta,phi, dtheta,dphi, rsintheta,y,v,x,z : single;
   i,j,m : integer;
   num_v, num_tri,n : integer;
   tmpvertex : ^D3DVERTEX;
   tmpTriangle, tmpTriangle2 : ^D3DTRIANGLE;
begin
     num_tri := (num_rings + 1) * num_sections *2;
     num_v := (num_rings+1) * num_sections + 2;


     lstVertices.clear;
     lstTriangles.clear;

     lstVertices.Capacity := num_v;
     lstTriangles.Capacity := num_tri;

     for i:=0 to num_tri -1 do
         lstTriangles.Add(tmpTriangle);

     tmpVertex := AllocMem(Sizeof(D3DVERTEX));
     tmpVertex^.x := 0;
     tmpvertex^.y := sy * sphere_r;
     tmpvertex^.z := 0;
     tmpvertex^.nx := 0;
     tmpvertex^.ny := 1;
     tmpVertex^.nz := 0;
     tmpVertex^.tu := 0;
     tmpVertex^.tv := 0;
     lstVertices.Add(tmpvertex);

     dTheta := (pi / (num_rings + 2));
     dphi := (2 * pi/ num_sections);


     theta := dTheta;
     for i:=0 to num_rings do
         begin
              y := sphere_r * cos(theta);
              v := theta / pi;
              rsintheta := sphere_r * sin(theta);
              phi := 0;
              for j:=0 to num_sections do
                  begin
                       x := rsintheta * sin(phi);
                       z := rsintheta * cos(phi);
                       tmpVertex := AllocMem(sizeof(D3DVERTEX));
                       tmpVertex^.x := sx * x;
                       tmpVertex^.y := sy * y;
                       tmpvertex^.z := sz * z;
                       tmpvertex^.nx := x / sphere_r;
                       tmpVertex^.ny := y / sphere_r;
                       tmpVertex^.nz := z / sphere_r;
                       tmpVertex^.tv := v;
                       tmpVertex^.tu := 1 - phi / (2*pi);
                       lstVertices.Add(tmpvertex);
                       phi := phi + dphi;
                  end;
              theta := theta + dTheta;
         end;

     tmpVertex := AllocMem(Sizeof(D3DVERTEX));
     tmpVertex^.x := 0;
     tmpvertex^.y := sy * -sphere_r;
     tmpvertex^.z := 0;
     tmpvertex^.nx := 0;
     tmpvertex^.ny := -1;
     tmpVertex^.nz := 0;
     tmpVertex^.tu := 0;
     tmpVertex^.tv := 1;
     lstVertices.Add(tmpvertex);

     if (num_sections < 30) then
        begin
             for i:=0 to num_sections -1 do
                 begin
                      tmpTriangle := AllocMem(sizeof(D3DTRIANGLE));
                      tmpTriangle^.v1 := 0;
                      tmpTriangle^.v2 := i + 1;
                      tmpTriangle^.v3 := 1 + ((i+1) mod num_sections);
                      tmpTriangle^.wFlags := D3DTRIFLAG_EDGEENABLE1 + D3DTRIFLAG_EDGEENABLE2;
                      if (i = 0) then
                              tmpTriangle^.wFlags := tmpTriangle^.wFlags + D3DTRIFLAG_START
                      else
                          tmpTriangle^.wFlags := tmpTriangle^.wFlags + D3DTRIFLAG_EVEN;

                      lstTriangles.Items[i]:=tmpTriangle;

                      tmpTriangle := AllocMem(sizeof(D3DTRIANGLE));
                      tmpTriangle^.v1 := num_v - 1;
                      tmpTriangle^.v2 := num_v - 2 - i;
                      tmpTriangle^.v3 := num_v - 2 - ((1+i) mod num_sections);
                      tmpTriangle^.wFlags := D3DTRIFLAG_EDGEENABLE1 + D3DTRIFLAG_EDGEENABLE2;
                      if (i = 0) then
                              tmpTriangle^.wFlags := tmpTriangle^.wFlags + D3DTRIFLAG_START
                      else
                          tmpTriangle^.wFlags := tmpTriangle^.wFlags + D3DTRIFLAG_EVEN;

                      lstTriangles.Items[num_tri - num_sections + i]:= tmpTriangle; //(tmpTriangle);
                 end;
        end
     else
         begin
              for i:=0 to num_sections-1 do
                  begin
                      tmpTriangle := AllocMem(sizeof(D3DTRIANGLE));
                      tmpTriangle^.v1 := 0;
                      tmpTriangle^.v2 := i + 1;
                      tmpTriangle^.v3 := 1 + ((i+1) mod num_sections);
                      tmpTriangle^.wFlags := D3DTRIFLAG_EDGEENABLE1 + D3DTRIFLAG_EDGEENABLE2;

                      lstTriangles.Items[i] := tmpTriangle;

                      tmpTriangle := AllocMem(sizeof(D3DTRIANGLE));
                      tmpTriangle^.v1 := num_v - 1;
                      tmpTriangle^.v2 := num_v - 2 - i;
                      tmpTriangle^.v3 := num_v - 2 - ((1+i) mod num_sections);
                      tmpTriangle^.wFlags := D3DTRIFLAG_EDGEENABLE1 + D3DTRIFLAG_EDGEENABLE2;

                      lstTriangles.Items[num_tri - num_sections + i] :=  tmpTriangle;
                  end;

         end;

     m := 1;
     n := num_sections;
     for i:=0 to num_rings - 1 do
       begin
         for j := 0 to num_sections - 1 do
             begin
                  tmpTriangle := AllocMem(sizeof(D3DTRIANGLE));
                  tmpTriangle^.v1 := m+j;
                  tmpTriangle^.v2 := m+num_sections + j;
                  tmpTriangle^.v3 := m+num_sections + ((j+1) mod num_sections);
                  tmpTriangle^.wFlags := D3DTRIFLAG_EDGEENABLETRIANGLE;

                  tmpTriangle^.wFlags := tmpTriangle^.wFlags + 1 + D3DTRIFLAG_EDGEENABLE1 + D3DTRIFLAG_EDGEENABLE2;

                  lstTriangles.Items[n] :=  tmpTriangle;

                  tmpTriangle2 := Allocmem(sizeof(D3DTRIANGLE));
                  tmpTriangle2^.v1 := tmpTriangle^.v1;
                  tmpTriangle2^.v2 := tmpTriangle^.v3;
                  tmpTriangle2^.v3 := m + ((j+1) mod num_sections);

                  tmpTriangle2^.wFlags := D3DTRIFLAG_EVEN + D3DTRIFLAG_EDGEENABLE2 + D3DTRIFLAG_EDGEENABLE3;

                  lstTriangles.Items[n+1] := tmpTriangle2;
                  n := n + 2;
             end;

         m := m + num_sections;
       end;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   SpeedButton1.Left := width - 120;
end;

end.
